@@ -469,7 +469,7 @@ class Redim(metaclass=AccessorPipelineMeta):
     def label(self, specs=None, **values):
         for k, v in values.items():
             dim = self._obj.get_dimension(k)
-            if dim and dim.name != dim.label and dim.label != v:
+            if dim and dim.label not in (dim.name, v):
                 raise ValueError('Cannot override an existing Dimension label')
         return self._redim('label', specs, **values)
 
@@ -518,7 +518,7 @@ class Opts(metaclass=AccessorPipelineMeta):
         keywords = {}
         groups = Options._option_groups if group is None else [group]
         backend = backend if backend else Store.current_backend
-        for group in groups:
+        for group in groups:  # noqa: PLR1704
             optsobj = Store.lookup_options(backend, self._obj, group,
                                            defaults=defaults)
             keywords = dict(keywords, **optsobj.kwargs)
