@@ -1,29 +1,23 @@
 from collections import OrderedDict, defaultdict
-try:
-    import itertools.izip as zip
-except ImportError:
-    pass
 
 import numpy as np
 
-from .interface import Interface, DataError
+from .. import util
 from ..dimension import dimension_name
 from ..element import Element
-from ..dimension import OrderedDict as cyODict
 from ..ndmapping import NdMapping, item_check, sorted_context
 from ..util import isscalar
-from .. import util
-
+from .interface import DataError, Interface
 
 
 class DictInterface(Interface):
     """
     Interface for simple dictionary-based dataset format. The dictionary
-    keys correspond to the column (i.e dimension) names and the values
+    keys correspond to the column (i.e. dimension) names and the values
     are collections representing the values in that column.
     """
 
-    types = (dict, OrderedDict, cyODict)
+    types = (dict, OrderedDict)
 
     datatype = 'dictionary'
 
@@ -36,7 +30,6 @@ class DictInterface(Interface):
 
     @classmethod
     def init(cls, eltype, data, kdims, vdims):
-        odict_types = (OrderedDict, cyODict)
         if kdims is None:
             kdims = eltype.kdims
         if vdims is None:
@@ -58,7 +51,7 @@ class DictInterface(Interface):
                     data = np.atleast_2d(data).T
             data = {k: data[:,i] for i,k in enumerate(dimensions)}
         elif isinstance(data, list) and data == []:
-            data = OrderedDict([(d, []) for d in dimensions])
+            data = dict([(d, []) for d in dimensions])
         elif isinstance(data, list) and isscalar(data[0]):
             if eltype._auto_indexable_1d:
                 data = {dimensions[0]: np.arange(len(data)), dimensions[1]: data}
@@ -115,7 +108,8 @@ class DictInterface(Interface):
 
         if not cls.expanded([vs for d, vs in unpacked if d in dimensions and not isscalar(vs)]):
             raise ValueError('DictInterface expects data to be of uniform shape.')
-        if isinstance(data, odict_types):
+        # OrderedDict can't be replaced with dict: https://github.com/holoviz/holoviews/pull/5925
+        if isinstance(data, OrderedDict):
             data.update(unpacked)
         else:
             data = OrderedDict(unpacked)
@@ -130,13 +124,13 @@ class DictInterface(Interface):
         not_found = [d for d in dimensions if d not in dataset.data]
         if not_found:
             raise DataError('Following columns specified as dimensions '
-                            'but not found in data: %s' % not_found, cls)
+                            f'but not found in data: {not_found}', cls)
         lengths = [(dim, 1 if isscalar(dataset.data[dim]) else len(dataset.data[dim]))
                    for dim in dimensions]
         if len({l for d, l in lengths if l > 1}) > 1:
             lengths = ', '.join(['%s: %d' % l for l in sorted(lengths)])
             raise DataError('Length of columns must be equal or scalar, '
-                            'columns have lengths: %s' % lengths, cls)
+                            f'columns have lengths: {lengths}', cls)
 
 
     @classmethod
@@ -147,7 +141,7 @@ class DictInterface(Interface):
         """
         if len(data) != 1:
             return data
-        key = list(data.keys())[0]
+        key = next(iter(data.keys()))
 
         if len(data[key]) == 1 and key in dataset.vdims:
             scalar = data[key][0]
@@ -195,7 +189,7 @@ class DictInterface(Interface):
         dim = dimension_name(dimension)
         data = list(dataset.data.items())
         data.insert(dim_pos, (dim, values))
-        return OrderedDict(data)
+        return dict(data)
 
     @classmethod
     def redim(cls, dataset, dimensions):
@@ -207,7 +201,7 @@ class DictInterface(Interface):
             elif k in all_dims:
                 k = dataset.get_dimension(k).name
             renamed.append((k, v))
-        return OrderedDict(renamed)
+        return dict(renamed)
 
 
     @classmethod
@@ -221,12 +215,12 @@ class DictInterface(Interface):
 
         template = datasets[0][1]
         dims = dimensions+template.dimensions()
-        return OrderedDict([(d.name, np.concatenate(columns[d.name])) for d in dims])
+        return dict([(d.name, np.concatenate(columns[d.name])) for d in dims])
 
 
     @classmethod
     def mask(cls, dataset, mask, mask_value=np.nan):
-        masked = OrderedDict(dataset.data)
+        masked = dict(dataset.data)
         for vd in dataset.vdims:
             new_array = np.copy(dataset.data[vd.name])
             new_array[mask] = mask_value
@@ -235,20 +229,22 @@ class DictInterface(Interface):
 
 
     @classmethod
-    def sort(cls, dataset, by=[], reverse=False):
+    def sort(cls, dataset, by=None, reverse=False):
+        if by is None:
+            by = []
         by = [dataset.get_dimension(d).name for d in by]
         if len(by) == 1:
             sorting = cls.values(dataset, by[0]).argsort()
         else:
             arrays = [dataset.dimension_values(d) for d in by]
             sorting = util.arglexsort(arrays)
-        return OrderedDict([(d, v if isscalar(v) else (v[sorting][::-1] if reverse else v[sorting]))
+        return dict([(d, v if isscalar(v) else (v[sorting][::-1] if reverse else v[sorting]))
                             for d, v in dataset.data.items()])
 
 
     @classmethod
     def range(cls, dataset, dimension):
-        dim = dataset.get_dimension(dimension)
+        dim = dataset.get_dimension(dimension, strict=True)
         column = dataset.data[dim.name]
         if isscalar(column):
             return column, column
@@ -272,7 +268,7 @@ class DictInterface(Interface):
 
     @classmethod
     def assign(cls, dataset, new_data):
-        data = OrderedDict(dataset.data)
+        data = dict(dataset.data)
         data.update(new_data)
         return data
 
@@ -280,7 +276,7 @@ class DictInterface(Interface):
     @classmethod
     def reindex(cls, dataset, kdims, vdims):
         dimensions = [dataset.get_dimension(d).name for d in kdims+vdims]
-        return OrderedDict([(d, dataset.dimension_values(d))
+        return dict([(d, dataset.dimension_values(d))
                             for d in dimensions])
 
 
@@ -308,9 +304,9 @@ class DictInterface(Interface):
         grouped_data = []
         for unique_key in util.unique_iterator(keys):
             mask = cls.select_mask(dataset, dict(zip(dimensions, unique_key)))
-            group_data = OrderedDict(((d.name, dataset.data[d.name] if isscalar(dataset.data[d.name])
+            group_data = dict((d.name, dataset.data[d.name] if isscalar(dataset.data[d.name])
                                        else dataset.data[d.name][mask])
-                                      for d in kdims+vdims))
+                                      for d in kdims+vdims)
             group_data = group_type(group_data, **group_kwargs)
             grouped_data.append((unique_key, group_data))
 
@@ -331,20 +327,22 @@ class DictInterface(Interface):
             return {d.name: np.array([], dtype=cls.dtype(dataset, d))
                     for d in dimensions}
         indexed = cls.indexed(dataset, selection)
-        data = OrderedDict()
+        data = {}
         for k, v in dataset.data.items():
             if k not in dimensions or isscalar(v):
                 data[k] = v
             else:
                 data[k] = v[selection_mask]
-        if indexed and len(list(data.values())[0]) == 1 and len(dataset.vdims) == 1:
+        if indexed and len(next(iter(data.values()))) == 1 and len(dataset.vdims) == 1:
             value = data[dataset.vdims[0].name]
             return value if isscalar(value) else value[0]
         return data
 
 
     @classmethod
-    def sample(cls, dataset, samples=[]):
+    def sample(cls, dataset, samples=None):
+        if samples is None:
+            samples = []
         mask = False
         for sample in samples:
             sample_mask = True
@@ -361,8 +359,8 @@ class DictInterface(Interface):
     def aggregate(cls, dataset, kdims, function, **kwargs):
         kdims = [dataset.get_dimension(d, strict=True).name for d in kdims]
         vdims = dataset.dimensions('value', label='name')
-        groups = cls.groupby(dataset, kdims, list, OrderedDict)
-        aggregated = OrderedDict([(k, []) for k in kdims+vdims])
+        groups = cls.groupby(dataset, kdims, list, dict)
+        aggregated = dict([(k, []) for k in kdims+vdims])
 
         dropped = []
         for key, group in groups:
@@ -400,7 +398,7 @@ class DictInterface(Interface):
         if isscalar(rows):
             rows = [rows]
 
-        new_data = OrderedDict()
+        new_data = {}
         for d, values in dataset.data.items():
             if d in cols:
                 if isscalar(values):
@@ -440,7 +438,7 @@ class DictInterface(Interface):
                 holes.append(subholes)
             return [holes]
         else:
-            return super(DictInterface, cls).holes(dataset)
+            return super().holes(dataset)
 
 
 Interface.register(DictInterface)
