@@ -1,26 +1,14 @@
-from __future__ import absolute_import
-
-import sys
-import datetime as dt
-
-from collections import OrderedDict, defaultdict
-
-try:
-    import itertools.izip as zip
-except ImportError:
-    pass
+from collections import defaultdict
 
 import numpy as np
 
-from .dictionary import DictInterface
-from .interface import Interface, DataError
+from .. import util
 from ..dimension import dimension_name
 from ..element import Element
-from ..dimension import OrderedDict as cyODict
 from ..ndmapping import NdMapping, item_check, sorted_context
-from .. import util
-from .interface import is_dask, dask_array_module, get_array_types
-
+from .dictionary import DictInterface
+from .interface import DataError, Interface
+from .util import dask_array_module, finite_range, get_array_types, is_dask
 
 
 class GridInterface(DictInterface):
@@ -28,7 +16,7 @@ class GridInterface(DictInterface):
     Interface for simple dictionary-based dataset format using a
     compressed representation that uses the cartesian product between
     key dimensions. As with DictInterface, the dictionary keys correspond
-    to the column (i.e dimension) names and the values are NumPy arrays
+    to the column (i.e. dimension) names and the values are NumPy arrays
     representing the values in that column.
 
     To use this compressed format, the key dimensions must be orthogonal
@@ -39,7 +27,7 @@ class GridInterface(DictInterface):
     longitudes can specify the position of NxM temperature samples.
     """
 
-    types = (dict, OrderedDict, cyODict)
+    types = (dict,)
 
     datatype = 'grid'
 
@@ -70,9 +58,9 @@ class GridInterface(DictInterface):
                 data = {d: v for d, v in zip(dimensions, data)}
         elif (isinstance(data, list) and data == []):
             if len(kdims) == 1:
-                data = OrderedDict([(d, []) for d in dimensions])
+                data = dict([(d, []) for d in dimensions])
             else:
-                data = OrderedDict([(d.name, np.array([])) for d in kdims])
+                data = dict([(d.name, np.array([])) for d in kdims])
                 if len(vdims) == 1:
                     data[vdims[0].name] = np.zeros((0, 0))
                 else:
@@ -83,11 +71,11 @@ class GridInterface(DictInterface):
         elif isinstance(data, np.ndarray):
             if data.shape == (0, 0) and len(vdims) == 1:
                 array = data
-                data = OrderedDict([(d.name, np.array([])) for d in kdims])
+                data = dict([(d.name, np.array([])) for d in kdims])
                 data[vdims[0].name] = array
             elif data.shape == (0, 0, len(vdims)):
                 array = data
-                data = OrderedDict([(d.name, np.array([])) for d in kdims])
+                data = dict([(d.name, np.array([])) for d in kdims])
                 data[vdim_tuple] = array
             else:
                 if data.ndim == 1:
@@ -113,7 +101,7 @@ class GridInterface(DictInterface):
         for dim in validate_dims:
             name = dimension_name(dim)
             if name not in data:
-                raise ValueError("Values for dimension %s not found" % dim)
+                raise ValueError(f"Values for dimension {dim} not found")
             if not isinstance(data[name], get_array_types()):
                 data[name] = np.array(data[name])
 
@@ -142,10 +130,10 @@ class GridInterface(DictInterface):
                             'match the expected dimensionality indicated '
                             'by the key dimensions. Expected %d-D array, '
                             'found %d-D array.' % (vdim, len(expected), len(shape)))
-            elif any((s!=e and (s+1)!=e) for s, e in zip(shape, valid_shape)):
-                raise error('Key dimension values and value array %s '
-                            'shapes do not match. Expected shape %s, '
-                            'actual shape: %s' % (vdim, valid_shape, shape), cls)
+            elif any((e not in (s, s + 1)) for s, e in zip(shape, valid_shape)):
+                raise error(f'Key dimension values and value array {vdim} '
+                            f'shapes do not match. Expected shape {valid_shape}, '
+                            f'actual shape: {shape}', cls)
         return data, {'kdims':kdims, 'vdims':vdims}, {}
 
 
@@ -170,12 +158,11 @@ class GridInterface(DictInterface):
         new_data[dim.name] = np.array(values)
         for vdim in vdims:
             arrays = [grid[vdim.name] for grid in grids]
-            shapes = set(arr.shape for arr in arrays)
+            shapes = {arr.shape for arr in arrays}
             if len(shapes) > 1:
                 raise DataError('When concatenating gridded data the shape '
-                                'of arrays must match. %s found that arrays '
-                                'along the %s dimension do not match.' %
-                                (cls.__name__, vdim.name))
+                                f'of arrays must match. {cls.__name__} found that arrays '
+                                f'along the {vdim.name} dimension do not match.')
             stack = dask_array_module().stack if any(is_dask(arr) for arr in arrays) else np.stack
             new_data[vdim.name] = stack(arrays, -1)
         return new_data
@@ -200,7 +187,7 @@ class GridInterface(DictInterface):
         if not_found and tuple(not_found) not in dataset.data:
             raise DataError("Supplied data does not contain specified "
                             "dimensions, the following dimensions were "
-                            "not found: %s" % repr(not_found), cls)
+                            f"not found: {not_found!r}", cls)
 
 
     @classmethod
@@ -242,7 +229,7 @@ class GridInterface(DictInterface):
         if gridded:
             return shape
         else:
-            return (np.product(shape, dtype=np.intp), len(dataset.dimensions()))
+            return (np.prod(shape, dtype=np.intp), len(dataset.dimensions()))
 
 
     @classmethod
@@ -260,9 +247,6 @@ class GridInterface(DictInterface):
                [ 2.5,  3.5,  4.5]])
         """
         coord = np.asarray(coord)
-        if sys.version_info.major == 2 and len(coord) and isinstance(coord[0], (dt.datetime, dt.date)):
-            # np.diff does not work on datetimes in python 2
-            coord = coord.astype('datetime64')
         if coord.shape[axis] == 0:
             return np.array([], dtype=coord.dtype)
         if coord.shape[axis] > 1:
@@ -313,7 +297,7 @@ class GridInterface(DictInterface):
 
 
     @classmethod
-    def canonicalize(cls, dataset, data, data_coords=None, virtual_coords=[]):
+    def canonicalize(cls, dataset, data, data_coords=None, virtual_coords=None):
         """
         Canonicalize takes an array of values as input and reorients
         and transposes it to match the canonical format expected by
@@ -328,6 +312,8 @@ class GridInterface(DictInterface):
         with a virtual integer index. This ensures these coordinates
         are not simply dropped.
         """
+        if virtual_coords is None:
+            virtual_coords = []
         if data_coords is None:
             data_coords = dataset.dimensions('key', label='name')[::-1]
 
@@ -337,7 +323,10 @@ class GridInterface(DictInterface):
         dropped = [dims.index(d) for d in dims
                    if d not in dataset.kdims+virtual_coords]
         if dropped:
-            data = np.squeeze(data, axis=tuple(dropped))
+            if len(dropped) == data.ndim:
+                data = data.flatten()
+            else:
+                data = np.squeeze(data, axis=tuple(dropped))
 
         if not any(cls.irregular(dataset, d) for d in dataset.kdims):
             inds = [dims.index(kd.name) for kd in dataset.kdims]
@@ -387,7 +376,7 @@ class GridInterface(DictInterface):
         selected = {}
         adjusted_inds = []
         all_scalar = True
-        for i, (kd, ind) in enumerate(zip(dataset.kdims[::-1], indices)):
+        for kd, ind in zip(dataset.kdims[::-1], indices):
             coords = cls.coords(dataset, kd.name, True)
             if np.isscalar(ind):
                 ind = [ind]
@@ -409,6 +398,17 @@ class GridInterface(DictInterface):
             selected[d.name] = arr[tuple(adjusted_inds)]
         return tuple(selected[d.name] for d in dataset.dimensions())
 
+    @classmethod
+    def persist(cls, dataset):
+        da = dask_array_module()
+        return {k: v.persist() if da and isinstance(v, da.Array) else v
+                for k, v in dataset.data.items()}
+
+    @classmethod
+    def compute(cls, dataset):
+        da = dask_array_module()
+        return {k: v.compute() if da and isinstance(v, da.Array) else v
+                for k, v in dataset.data.items()}
 
     @classmethod
     def values(cls, dataset, dim, expanded=True, flat=True, compute=True,
@@ -445,9 +445,8 @@ class GridInterface(DictInterface):
 
         invalid = [d for d in dimensions if dataset.data[d.name].ndim > 1]
         if invalid:
-            if len(invalid) == 1: invalid = "'%s'" % invalid[0]
-            raise ValueError("Cannot groupby irregularly sampled dimension(s) %s."
-                             % invalid)
+            if len(invalid) == 1: invalid = f"'{invalid[0]}'"
+            raise ValueError(f"Cannot groupby irregularly sampled dimension(s) {invalid}.")
 
         # Update the kwargs appropriately for Element group types
         group_kwargs = {}
@@ -498,7 +497,7 @@ class GridInterface(DictInterface):
 
     @classmethod
     def key_select_mask(cls, dataset, values, ind):
-        if util.pd and values.dtype.kind == 'M':
+        if values.dtype.kind == 'M':
             ind = util.parse_datetime_selection(ind)
         if isinstance(ind, tuple):
             ind = slice(*ind)
@@ -512,7 +511,7 @@ class GridInterface(DictInterface):
                 mask &= values < ind.stop
             # Expand empty mask
             if mask is True:
-                mask = np.ones(values.shape, dtype=np.bool)
+                mask = np.ones(values.shape, dtype=np.bool_)
         elif isinstance(ind, (set, list)):
             iter_slcs = []
             for ik in ind:
@@ -526,7 +525,7 @@ class GridInterface(DictInterface):
             index_mask = values == ind
             if (dataset.ndims == 1 or dataset._binned) and np.sum(index_mask) == 0:
                 data_index = np.argmin(np.abs(values - ind))
-                mask = np.zeros(len(values), dtype=np.bool)
+                mask = np.zeros(len(values), dtype=np.bool_)
                 mask[data_index] = True
             else:
                 mask = index_mask
@@ -537,6 +536,9 @@ class GridInterface(DictInterface):
 
     @classmethod
     def select(cls, dataset, selection_mask=None, **selection):
+        if selection_mask is not None:
+            raise ValueError(f"Masked selections currently not supported for {cls.__name__}.")
+
         dimensions = dataset.kdims
         val_dims = [vdim for vdim in dataset.vdims if vdim in selection]
         if val_dims:
@@ -555,9 +557,8 @@ class GridInterface(DictInterface):
             if irregular:
                 if np.isscalar(ind) or isinstance(ind, (set, list)):
                     raise IndexError("Indexing not supported for irregularly "
-                                     "sampled data. %s value along %s dimension."
-                                     "must be a slice or 2D boolean mask."
-                                     % (ind, dim))
+                                     f"sampled data. {ind} value along {dim} dimension."
+                                     "must be a slice or 2D boolean mask.")
                 mask = mask.max(axis=i)
             elif dataset._binned:
                 edges = cls.coords(dataset, dim, False, edges=True)
@@ -565,13 +566,13 @@ class GridInterface(DictInterface):
                 if np.isscalar(ind):
                     emin, emax = edges.min(), edges.max()
                     if ind < emin:
-                        raise IndexError("Index %s less than lower bound "
-                                         "of %s for %s dimension." % (ind, emin, dim))
+                        raise IndexError(f"Index {ind} less than lower bound "
+                                         f"of {emin} for {dim} dimension.")
                     elif ind >= emax:
-                        raise IndexError("Index %s more than or equal to upper bound "
-                                         "of %s for %s dimension." % (ind, emax, dim))
+                        raise IndexError(f"Index {ind} more than or equal to upper bound "
+                                         f"of {emax} for {dim} dimension.")
                     idx = max([np.digitize([ind], edges)[0]-1, 0])
-                    mask = np.zeros(len(values), dtype=np.bool)
+                    mask = np.zeros(len(values), dtype=np.bool_)
                     mask[idx] = True
                     values = edges[idx:idx+2]
                 elif len(inds):
@@ -620,7 +621,7 @@ class GridInterface(DictInterface):
     def mask(cls, dataset, mask, mask_val=np.nan):
         mask = cls.canonicalize(dataset, mask)
         packed = cls.packed(dataset)
-        masked = OrderedDict(dataset.data)
+        masked = dict(dataset.data)
         if packed:
             masked = dataset.data[packed].copy()
             try:
@@ -640,10 +641,12 @@ class GridInterface(DictInterface):
 
 
     @classmethod
-    def sample(cls, dataset, samples=[]):
+    def sample(cls, dataset, samples=None):
         """
         Samples the gridded data into dataset of samples.
         """
+        if samples is None:
+            samples = []
         ndims = dataset.ndims
         dimensions = dataset.dimensions(label='name')
         arrays = [dataset.data[vdim.name] for vdim in dataset.vdims]
@@ -741,7 +744,9 @@ class GridInterface(DictInterface):
 
 
     @classmethod
-    def sort(cls, dataset, by=[], reverse=False):
+    def sort(cls, dataset, by=None, reverse=False):
+        if by is None:
+            by = []
         if not by or by in [dataset.kdims, dataset.dimensions()]:
             return dataset.data
         else:
@@ -776,33 +781,34 @@ class GridInterface(DictInterface):
 
     @classmethod
     def range(cls, dataset, dimension):
+        dimension = dataset.get_dimension(dimension, strict=True)
         if dataset._binned and dimension in dataset.kdims:
             expanded = cls.irregular(dataset, dimension)
-            column = cls.coords(dataset, dimension, expanded=expanded, edges=True)
+            array = cls.coords(dataset, dimension, expanded=expanded, edges=True)
         else:
-            column = cls.values(dataset, dimension, expanded=False, flat=False)
+            array = cls.values(dataset, dimension, expanded=False, flat=False)
+
+        if dimension.nodata is not None:
+            array = cls.replace_value(array, dimension.nodata)
 
         da = dask_array_module()
-        if column.dtype.kind == 'M':
-            dmin, dmax = column.min(), column.max()
-            if da and isinstance(column, da.Array):
-                return da.compute(dmin, dmax)
-            return dmin, dmax
-        elif len(column) == 0:
-            return np.NaN, np.NaN
+        if len(array) == 0:
+            return np.nan, np.nan
+
+        if array.dtype.kind == 'M':
+            dmin, dmax = array.min(), array.max()
         else:
             try:
-                dmin, dmax = (np.nanmin(column), np.nanmax(column))
-                if da and isinstance(column, da.Array):
-                    return da.compute(dmin, dmax)
-                return dmin, dmax
+                dmin, dmax = (np.nanmin(array), np.nanmax(array))
             except TypeError:
-                column.sort()
-                return column[0], column[-1]
+                return np.nan, np.nan
+        if da and isinstance(array, da.Array):
+            return finite_range(array, *da.compute(dmin, dmax))
+        return finite_range(array, dmin, dmax)
 
     @classmethod
     def assign(cls, dataset, new_data):
-        data = OrderedDict(dataset.data)
+        data = dict(dataset.data)
         for k, v in new_data.items():
             if k in dataset.kdims:
                 coords = cls.coords(dataset, k)
@@ -813,6 +819,6 @@ class GridInterface(DictInterface):
                 data[k] = cls.canonicalize(dataset, v)
         return data
 
-            
+
 
 Interface.register(GridInterface)
