@@ -1,9 +1,7 @@
-from __future__ import absolute_import, division, unicode_literals
-
 from collections import defaultdict
 
-import param
 import numpy as np
+import param
 
 from ...core import util
 from ...element import Contours, Polygons
@@ -12,10 +10,14 @@ from .callbacks import PolyDrawCallback, PolyEditCallback
 from .element import ColorbarPlot, LegendPlot, OverlayPlot
 from .selection import BokehOverlaySelectionDisplay
 from .styles import (
-    expand_batched_style, base_properties, line_properties, fill_properties,
-    mpl_to_bokeh, validate
+    base_properties,
+    expand_batched_style,
+    fill_properties,
+    line_properties,
+    mpl_to_bokeh,
+    validate,
 )
-from .util import bokeh_version, multi_polygons_data
+from .util import multi_polygons_data
 
 
 class PathPlot(LegendPlot, ColorbarPlot):
@@ -29,7 +31,7 @@ class PathPlot(LegendPlot, ColorbarPlot):
 
     # Deprecated options
 
-    color_index = param.ClassSelector(default=None, class_=(util.basestring, int),
+    color_index = param.ClassSelector(default=None, class_=(str, int),
                                       allow_None=True, doc="""
         Deprecated in favor of color style mapping, e.g. `color=dim('color')`""")
 
@@ -42,7 +44,7 @@ class PathPlot(LegendPlot, ColorbarPlot):
 
     def _element_transform(self, transform, element, ranges):
         if isinstance(element, Contours):
-            data = super(PathPlot, self)._element_transform(transform, element, ranges)
+            data = super()._element_transform(transform, element, ranges)
             new_data = []
             for d in data:
                 if isinstance(d, np.ndarray) and len(d) == 1:
@@ -74,13 +76,13 @@ class PathPlot(LegendPlot, ColorbarPlot):
         for k, v in self.overlay_dims.items():
             dim = util.dimension_sanitizer(k.name)
             if dim not in data:
-                data[dim] = [v for _ in range(len(list(data.values())[0]))]
+                data[dim] = [v] * len(next(iter(data.values())))
 
 
     def get_data(self, element, ranges, style):
         color = style.get('color', None)
         cdim = None
-        if isinstance(color, util.basestring) and not validate('color', color):
+        if isinstance(color, str) and not validate('color', color):
             cdim = element.get_dimension(color)
         elif self.color_index is not None:
             cdim = element.get_dimension(self.color_index)
@@ -88,7 +90,7 @@ class PathPlot(LegendPlot, ColorbarPlot):
         scalar = element.interface.isunique(element, cdim, per_geom=True) if cdim else False
         style_mapping = {
             (s, v) for s, v in style.items() if (s not in self._nonvectorized_styles) and
-            ((isinstance(v, util.basestring) and v in element) or isinstance(v, dim)) and
+            ((isinstance(v, str) and v in element) or isinstance(v, dim)) and
             not (not isinstance(v, dim) and v == color and s == 'color')}
         mapping = dict(self._mapping)
 
@@ -147,7 +149,7 @@ class PathPlot(LegendPlot, ColorbarPlot):
         zorders = self._updated_zorders(element)
         for (key, el), zorder in zip(element.data.items(), zorders):
             el_opts = self.lookup_options(el, 'plot').options
-            self.param.set_param(**{k: v for k, v in el_opts.items()
+            self.param.update(**{k: v for k, v in el_opts.items()
                                     if k not in OverlayPlot._propagate_options})
             style = self.lookup_options(el, 'style')
             style = style.max_cycles(len(self.ordering))[zorder]
@@ -161,7 +163,7 @@ class PathPlot(LegendPlot, ColorbarPlot):
                 continue
 
             # Apply static styles
-            nvals = len(list(eldata.values())[0])
+            nvals = len(next(iter(eldata.values())))
             sdata, smapping = expand_batched_style(style, self._batched_style_opts,
                                                    elmapping, nvals)
             elmapping.update({k: v for k, v in smapping.items() if k not in elmapping})
@@ -182,7 +184,7 @@ class ContourPlot(PathPlot):
 
     # Deprecated options
 
-    color_index = param.ClassSelector(default=0, class_=(util.basestring, int),
+    color_index = param.ClassSelector(default=0, class_=(str, int),
                                       allow_None=True, doc="""
         Deprecated in favor of color style mapping, e.g. `color=dim('color')`""")
 
@@ -190,7 +192,7 @@ class ContourPlot(PathPlot):
     _nonvectorized_styles = base_properties + ['cmap']
 
     def __init__(self, *args, **params):
-        super(ContourPlot, self).__init__(*args, **params)
+        super().__init__(*args, **params)
         self._has_holes = None
 
     def _hover_opts(self, element):
@@ -210,13 +212,10 @@ class ContourPlot(PathPlot):
 
         interface = element.interface
         scalar_kwargs = {'per_geom': True} if interface.multi else {}
-        npath = len([vs for vs in data.values()][0])
         for d in element.vdims:
             dim = util.dimension_sanitizer(d.name)
             if dim not in data:
-                if element.level is not None:
-                    data[dim] = np.full(npath, element.level)
-                elif interface.isunique(element, d, **scalar_kwargs):
+                if interface.isunique(element, d, **scalar_kwargs):
                     data[dim] = element.dimension_values(d, expanded=False)
                 else:
                     data[dim] = element.split(datatype='array', dimensions=[d])
@@ -224,7 +223,7 @@ class ContourPlot(PathPlot):
         for k, v in self.overlay_dims.items():
             dim = util.dimension_sanitizer(k.name)
             if dim not in data:
-                data[dim] = [v for _ in range(len(list(data.values())[0]))]
+                data[dim] = [v] * len(next(iter(data.values())))
 
     def get_data(self, element, ranges, style):
         if self._has_holes is None:
@@ -239,7 +238,7 @@ class ContourPlot(PathPlot):
             element = element.clone([element.data], datatype=type(element).datatype)
 
         if self.static_source:
-            data = dict()
+            data = {}
             xs = self.handles['cds'].data['xs']
         else:
             if has_holes:
@@ -257,8 +256,6 @@ class ContourPlot(PathPlot):
         if (((isinstance(color, dim) and color.applies(element)) or color in element) or
             (isinstance(fill_color, dim) and fill_color.applies(element)) or fill_color in element):
             cdim = None
-        elif None not in [element.level, self.color_index] and element.vdims:
-            cdim = element.vdims[0]
         else:
             cidx = self.color_index+2 if isinstance(self.color_index, int) else self.color_index
             cdim = element.get_dimension(cidx)
@@ -266,17 +263,13 @@ class ContourPlot(PathPlot):
         if cdim is None:
             return data, mapping, style
 
-        ncontours = len(xs)
         dim_name = util.dimension_sanitizer(cdim.name)
-        if element.level is not None:
-            values = np.full(ncontours, float(element.level))
-        else:
-            values = element.dimension_values(cdim, expanded=False)
+        values = element.dimension_values(cdim, expanded=False)
         data[dim_name] = values
 
         factors = None
-        if cdim.name in ranges and 'factors' in ranges[cdim.name]:
-            factors = ranges[cdim.name]['factors']
+        if cdim.label in ranges and 'factors' in ranges[cdim.label]:
+            factors = ranges[cdim.label]['factors']
         elif values.dtype.kind in 'SUO' and len(values):
             if isinstance(values[0], np.ndarray):
                 values = np.concatenate(values)
@@ -284,8 +277,7 @@ class ContourPlot(PathPlot):
         cmapper = self._get_colormapper(cdim, element, ranges, style, factors)
         mapping[self._color_style] = {'field': dim_name, 'transform': cmapper}
         if self.show_legend:
-            legend_prop = 'legend_field' if bokeh_version >= '1.3.5' else 'legend'
-            mapping[legend_prop] = dim_name
+            mapping['legend_field'] = dim_name
         return data, mapping, style
 
     def _init_glyph(self, plot, mapping, properties):
