@@ -62,10 +62,12 @@ from ...core.util import (
 from ...util.warnings import warn
 from ..util import dim_axis_label
 
-bokeh_version = Version(Version(bokeh.__version__).base_version)
-bokeh32 = bokeh_version >= Version("3.2")
-bokeh33 = bokeh_version >= Version("3.3")
-bokeh34 = bokeh_version >= Version("3.4")
+BOKEH_VERSION = Version(bokeh.__version__).release
+BOKEH_GE_3_2_0 = BOKEH_VERSION >= (3, 2, 0)
+BOKEH_GE_3_3_0 = BOKEH_VERSION >= (3, 3, 0)
+BOKEH_GE_3_4_0 = BOKEH_VERSION >= (3, 4, 0)
+BOKEH_GE_3_5_0 = BOKEH_VERSION >= (3, 5, 0)
+BOKEH_GE_3_6_0 = BOKEH_VERSION >= (3, 6, 0)
 
 TOOL_TYPES = {
     'pan': tools.PanTool,
@@ -863,11 +865,19 @@ def hold_render(f):
     """
     def wrapper(self, *args, **kwargs):
         hold = self.state.hold_render
+        doc = self.state.document
         self.state.hold_render = True
-        try:
-            return f(self, *args, **kwargs)
-        finally:
-            self.state.hold_render = hold
+        if doc:
+            with doc.models.freeze():
+                try:
+                    return f(self, *args, **kwargs)
+                finally:
+                    self.state.hold_render = hold
+        else:
+            try:
+                return f(self, *args, **kwargs)
+            finally:
+                self.state.hold_render = hold
     return wrapper
 
 
