@@ -8,24 +8,32 @@ import numpy as np
 try:
     import spatialpandas
     from spatialpandas.geometry import (
-        MultiPolygonArray, LineDtype, PointDtype, PolygonDtype,
-        MultiLineDtype, MultiPointDtype, MultiPolygonDtype
+        LineDtype,
+        MultiLineDtype,
+        MultiPointDtype,
+        MultiPolygonArray,
+        MultiPolygonDtype,
+        PointDtype,
+        PolygonDtype,
     )
-except Exception:
+except ImportError:
     spatialpandas = None
 
 try:
     import dask.dataframe as dd
-except Exception:
+except ImportError:
     dd = None
 
 from holoviews.core.data import (
-    Dataset, SpatialPandasInterface, DaskSpatialPandasInterface
+    DaskSpatialPandasInterface,
+    Dataset,
+    SpatialPandasInterface,
 )
 from holoviews.core.data.interface import DataError
 from holoviews.element import Path, Points, Polygons
 from holoviews.element.comparison import ComparisonTestCase
 
+from ...utils import dask_switcher
 from .test_multiinterface import GeomTests
 
 
@@ -229,9 +237,9 @@ class SpatialPandasTest(GeomTests, RoundTripTests):
             [[[0, 0, 1, 0, 2, 2, -1, 4, 0, 0],         # Filled quadrilateral (CCW order)
               [0.5, 1,  1, 2,  1.5, 1.5,  0.5, 1],     # Triangular hole (CW order)
               [0, 2, 0, 2.5, 0.5, 2.5, 0.5, 2, 0, 2]], # Rectangular hole (CW order)
-             
+
              [[-0.5, 3, 1.5, 3, 1.5, 4, -0.5, 3]],],   # Filled triangle
-            
+
             # Second Element
             [[[1.25, 0, 1.25, 2, 4, 2, 4, 0, 1.25, 0],          # Filled rectangle (CCW order)
               [1.5, 0.25, 3.75, 0.25, 3.75, 1.75, 1.5, 1.75, 1.5, 0.25]],]
@@ -252,6 +260,7 @@ class DaskSpatialPandasTest(GeomTests, RoundTripTests):
 
     __test__ = True
 
+    @dask_switcher(query=False, extras=["spatialpandas.dask"])
     def setUp(self):
         if spatialpandas is None:
             raise SkipTest('DaskSpatialPandasInterface requires spatialpandas, skipping tests')
