@@ -1,14 +1,13 @@
 import operator
-import sys
-
 from types import BuiltinFunctionType, BuiltinMethodType, FunctionType, MethodType
 
 import numpy as np
+import pandas as pd
 import param
 
 from ..core.data import PandasInterface
 from ..core.dimension import Dimension
-from ..core.util import flatten, pd, resolve_dependent_value, unique_iterator
+from ..core.util import flatten, resolve_dependent_value, unique_iterator
 
 
 def _maybe_map(numpy_fn):
@@ -23,14 +22,13 @@ def _maybe_map(numpy_fn):
                                       index=s.index))
             else:
                 return map_fn(lambda s: numpy_fn(s, *args, **kwargs))
+        elif series_like:
+            return type(values)(
+                numpy_fn(values, *args, **kwargs),
+                index=values.index,
+            )
         else:
-            if series_like:
-                return type(values)(
-                    numpy_fn(values, *args, **kwargs),
-                    index=values.index,
-                )
-            else:
-                return numpy_fn(values, *args, **kwargs)
+            return numpy_fn(values, *args, **kwargs)
     return fn
 
 
@@ -69,7 +67,7 @@ def lognorm(values, min=None, max=None):
     return (np.log(values) - min) / (max-min)
 
 
-class iloc(object):
+class iloc:
     """Implements integer array indexing for dim expressions.
     """
 
@@ -84,7 +82,28 @@ class iloc(object):
         return dim(self.expr, self)
 
     def __call__(self, values):
-        return values[self.index]
+        if isinstance(values, (pd.Series, pd.DataFrame)):
+            return values.iloc[resolve_dependent_value(self.index)]
+        else:
+            return values[resolve_dependent_value(self.index)]
+
+
+class loc:
+    """Implements loc for dim expressions.
+    """
+
+    __name__ = 'loc'
+
+    def __init__(self, dim_expr):
+        self.expr = dim_expr
+        self.index = slice(None)
+
+    def __getitem__(self, index):
+        self.index = index
+        return dim(self.expr, self)
+
+    def __call__(self, values):
+        return values.loc[resolve_dependent_value(self.index)]
 
 
 @_maybe_map
@@ -151,8 +170,8 @@ def categorize(values, categories, default=None):
     return result
 
 
-digitize = _maybe_map(np.digitize)
 isin = _maybe_map(np.isin)
+digitize = _maybe_map(np.digitize)
 astype = _maybe_map(np.asarray)
 round_ = _maybe_map(np.round)
 
@@ -161,13 +180,15 @@ def _python_isin(array, values):
 
 python_isin = _maybe_map(_python_isin)
 
+# Type of numpy function like np.max changed in Numpy 1.25
+# from function to a numpy._ArrayFunctionDispatcher.
 function_types = (
     BuiltinFunctionType, BuiltinMethodType, FunctionType,
-    MethodType, np.ufunc, iloc
+    MethodType, np.ufunc, iloc, loc, type(np.max)
 )
 
 
-class dim(object):
+class dim:
     """
     dim transform objects are a way to express deferred transforms on
     Datasets. dim transforms support all mathematical and bitwise
@@ -196,6 +217,7 @@ class dim(object):
         astype: 'astype',
         round_: 'round',
         iloc: 'iloc',
+        loc: 'loc',
     }
 
     _numpy_funcs = {
@@ -218,7 +240,7 @@ class dim(object):
 
     def __init__(self, obj, *args, **kwargs):
         from panel.widgets import Widget
-        ops = []
+        self.ops = []
         self._ns = np.ndarray
         self.coerce = kwargs.get('coerce', True)
         if isinstance(obj, str):
@@ -231,7 +253,7 @@ class dim(object):
             self.dimension = obj.param.value
         else:
             self.dimension = obj.dimension
-            ops = obj.ops
+            self.ops = obj.ops
         if args:
             fn = args[0]
         else:
@@ -240,10 +262,9 @@ class dim(object):
             if not (isinstance(fn, function_types+(str,)) or
                     any(fn in funcs for funcs in self._all_funcs)):
                 raise ValueError('Second argument must be a function, '
-                                 'found %s type' % type(fn))
-            ops = ops + [{'args': args[1:], 'fn': fn, 'kwargs': kwargs,
+                                 f'found {type(fn)} type')
+            self.ops = self.ops + [{'args': args[1:], 'fn': fn, 'kwargs': kwargs,
                           'reverse': kwargs.pop('reverse', False)}]
-        self.ops = ops
 
     def __getstate__(self):
         return self.__dict__
@@ -259,10 +280,10 @@ class dim(object):
     def __call__(self, *args, **kwargs):
         if (not self.ops or not isinstance(self.ops[-1]['fn'], str) or
             'accessor' not in self.ops[-1]['kwargs']):
-            raise ValueError("Cannot call method on %r expression. "
-                             "Only methods accessed via namspaces, "
+            raise ValueError(f"Cannot call method on {self!r} expression. "
+                             "Only methods accessed via namespaces, "
                              "e.g. dim(...).df or dim(...).xr), "
-                             "can be called. " % self)
+                             "can be called.")
         op = self.ops[-1]
         if op['fn'] == 'str':
             new_op = dict(op, fn=astype, args=(str,), kwargs={})
@@ -285,7 +306,7 @@ class dim(object):
                 # transform itself, so set namespace to None
                 ns = None
         extras = {ns_attr for ns_attr in dir(ns) if not ns_attr.startswith('_')}
-        if attr in extras and attr not in super(dim, self).__dir__():
+        if attr in extras and attr not in super().__dir__():
             return type(self)(self, attr, accessor=True)
         else:
             return super().__getattribute__(attr)
@@ -296,7 +317,7 @@ class dim(object):
             ns = getattr(ns, self._current_accessor)
         extras = {attr for attr in dir(ns) if not attr.startswith('_')}
         try:
-            return sorted(set(super(dim, self).__dir__()) | extras)
+            return sorted(set(super().__dir__()) | extras)
         except Exception:
             return sorted(set(dir(type(self))) | set(self.__dict__) | extras)
 
@@ -327,24 +348,34 @@ class dim(object):
 
     @property
     def params(self):
+        from panel.widgets.base import Widget
         params = {}
         for op in self.ops:
             op_args = list(op['args'])+list(op['kwargs'].values())
+            if hasattr(op['fn'], 'index'):
+                # Special case for loc and iloc to check for parameters
+                op_args += [op['fn'].index]
             op_args = flatten(op_args)
             for op_arg in op_args:
-                if 'panel' in sys.modules:
-                    from panel.widgets.base import Widget
-                    if isinstance(op_arg, Widget):
-                        op_arg = op_arg.param.value
+                if isinstance(op_arg, Widget):
+                    op_arg = op_arg.param.value
                 if isinstance(op_arg, dim):
                     params.update(op_arg.params)
                 elif isinstance(op_arg, slice):
-                    if isinstance(op_arg.start, param.Parameter):
-                        params[op_arg.start.name+str(id(op_arg.start))] = op_arg.start
-                    if isinstance(op_arg.stop, param.Parameter):
-                        params[op_arg.stop.name+str(id(op_arg.stop))] = op_arg.stop
-                    if isinstance(op_arg.step, param.Parameter):
-                        params[op_arg.step.name+str(id(op_arg.step))] = op_arg.step
+                    (start, stop, step) = (op_arg.start, op_arg.stop, op_arg.step)
+                    if isinstance(start, Widget):
+                        start = start.param.value
+                    if isinstance(stop, Widget):
+                        stop = stop.param.value
+                    if isinstance(step, Widget):
+                        step = step.param.value
+
+                    if isinstance(start, param.Parameter):
+                        params[start.name+str(id(start))] = start
+                    if isinstance(stop, param.Parameter):
+                        params[stop.name+str(id(stop))] = stop
+                    if isinstance(step, param.Parameter):
+                        params[step.name+str(id(step))] = step
                 if (isinstance(op_arg, param.Parameter) and
                     isinstance(op_arg.owner, param.Parameterized)):
                     params[op_arg.name+str(id(op_arg))] = op_arg
@@ -583,12 +614,6 @@ class dim(object):
                     dataset, flat, expanded, ranges, all_values,
                     keep_index, compute, strict
                 )
-            elif isinstance(arg, slice):
-                arg = slice(
-                    resolve_dependent_value(arg.start),
-                    resolve_dependent_value(arg.stop),
-                    resolve_dependent_value(arg.step)
-                )
             arg = resolve_dependent_value(arg)
             fn_args.append(arg)
         fn_kwargs = {}
@@ -598,12 +623,6 @@ class dim(object):
                     dataset, flat, expanded, ranges, all_values,
                     keep_index, compute, strict
                 )
-            elif isinstance(v, slice):
-                v = slice(
-                    resolve_dependent_value(v.start),
-                    resolve_dependent_value(v.stop),
-                    resolve_dependent_value(v.step)
-                )
             fn_kwargs[k] = resolve_dependent_value(v)
         args = tuple(fn_args[::-1] if op['reverse'] else fn_args)
         kwargs = dict(fn_kwargs)
@@ -618,9 +637,8 @@ class dim(object):
             if method is None:
                 mtype = 'attribute' if accessor else 'method'
                 raise AttributeError(
-                    "%r could not be applied to '%r', '%s' %s "
-                    "does not exist on %s type."
-                    % (self, dataset, fn, mtype, type(data).__name__)
+                    f"{self!r} could not be applied to '{dataset!r}', '{fn}' {mtype} "
+                    f"does not exist on {type(data).__name__} type."
                 )
             if accessor:
                 data = method
@@ -654,7 +672,7 @@ class dim(object):
         """
         return data
 
-    def apply(self, dataset, flat=False, expanded=None, ranges={}, all_values=False,
+    def apply(self, dataset, flat=False, expanded=None, ranges=None, all_values=False,
               keep_index=False, compute=True, strict=False):
         """Evaluates the transform on the supplied dataset.
 
@@ -679,27 +697,28 @@ class dim(object):
         """
         from ..element import Graph
 
+        if ranges is None:
+            ranges = {}
+
         dimension = self.dimension
         if expanded is None:
             expanded = not ((dataset.interface.gridded and dimension in dataset.kdims) or
                             (dataset.interface.multi and dataset.interface.isunique(dataset, dimension, True)))
 
         if not self.applies(dataset) and (not isinstance(dataset, Graph) or not self.applies(dataset.nodes)):
-            raise KeyError("One or more dimensions in the expression %r "
-                           "could not resolve on '%s'. Ensure all "
+            raise KeyError(f"One or more dimensions in the expression {self!r} "
+                           f"could not resolve on '{dataset}'. Ensure all "
                            "dimensions referenced by the expression are "
-                           "present on the supplied object." % (self, dataset))
+                           "present on the supplied object.")
         if not self.interface_applies(dataset, coerce=self.coerce):
             if self.coerce:
-                raise ValueError("The expression %r assumes a %s-like "
-                                 "API but the dataset contains %s data "
-                                 "and cannot be coerced." %
-                                 (self, self.namespace, dataset.interface.datatype))
+                raise ValueError(f"The expression {self!r} assumes a {self.namespace}-like "
+                                 f"API but the dataset contains {dataset.interface.datatype} data "
+                                 "and cannot be coerced.")
             else:
-                raise ValueError("The expression %r assumes a %s-like "
-                                 "API but the dataset contains %s data "
-                                 "and coercion is disabled." %
-                                 (self, self.namespace, dataset.interface.datatype))
+                raise ValueError(f"The expression {self!r} assumes a {self.namespace}-like "
+                                 f"API but the dataset contains {dataset.interface.datatype} data "
+                                 "and coercion is disabled.")
 
         if isinstance(dataset, Graph):
             if dimension in dataset.kdims and all_values:
@@ -743,7 +762,7 @@ class dim(object):
         return data
 
     def __repr__(self):
-        op_repr = "'%s'" % self.dimension
+        op_repr = f"'{self.dimension}'"
         accessor = False
         for i, o in enumerate(self.ops):
             if i == 0:
@@ -759,7 +778,7 @@ class dim(object):
             prev_accessor = accessor
             accessor = kwargs.pop('accessor', None)
             kwargs = sorted(kwargs.items(), key=operator.itemgetter(0))
-            kwargs = '%s' % ', '.join(['%s=%r' % item for item in kwargs]) if kwargs else ''
+            kwargs = ', '.join(['{}={!r}'.format(*item) for item in kwargs]) if kwargs else ''
             if fn in self._binary_funcs:
                 fn_name = self._binary_funcs[o['fn']]
                 if o['reverse']:
@@ -789,7 +808,9 @@ class dim(object):
                     fn_name = self._numpy_funcs[fn]
                     format_string = prev+').{fn}('
                 elif isinstance(fn, iloc):
-                    format_string = prev+').iloc[{0}]'.format(repr(fn.index))
+                    format_string = prev+f').iloc[{fn.index!r}]'
+                elif isinstance(fn, loc):
+                    format_string = prev+f').loc[{fn.index!r}]'
                 elif fn in self._custom_funcs:
                     fn_name = self._custom_funcs[fn]
                     format_string = prev+').{fn}('
@@ -829,7 +850,7 @@ class dim(object):
             if op_repr.count('(') - op_repr.count(')') > 0:
                 op_repr += ')'
         if not self.ops:
-            op_repr = 'dim({repr})'.format(repr=op_repr)
+            op_repr = f'dim({op_repr})'
         if op_repr.count('(') - op_repr.count(')') > 0:
             op_repr += ')'
         return op_repr
@@ -871,6 +892,9 @@ class df_dim(dim):
                      if dataset.interface.multi == intfc.multi]
         return dataset.clone(datatype=datatypes)
 
+    @property
+    def loc(self):
+        return loc(self)
 
 
 class xr_dim(dim):
@@ -889,7 +913,7 @@ class xr_dim(dim):
             import xarray as xr
         except ImportError:
             raise ImportError("XArray could not be imported, dim().xr "
-                              "requires the xarray to be available.")
+                              "requires the xarray to be available.") from None
         super().__init__(obj, *args, **kwargs)
         self._ns = xr.DataArray
 
